from datetime import datetime
from typing import List, Optional


class AnalogClass:
    ID_analog_math: int
    ID_analog: int
    type: str
    math: str
    created_at: datetime
    updated_at: datetime
    ID_aux: int
    max_voltage: int
    max_value: int

    def __init__(self, ID_analog_math: int, ID_analog: int, type: str, math: str, created_at: datetime,
                 updated_at: datetime, ID_aux: int, max_voltage: int, max_value: int) -> None:
        self.ID_analog_math = ID_analog_math
        self.ID_analog = ID_analog
        self.type = type
        self.math = math
        self.created_at = created_at
        self.updated_at = updated_at
        self.ID_aux = ID_aux
        self.max_voltage = max_voltage
        self.max_value = max_value


class Aux:
    ID_AUX_station: int
    ID_station: int
    title: str
    stationTitle: str
    output: int
    type: int
    expansion: int
    local: int
    settings: None
    created_at: datetime
    updated_at: datetime
    ID_aux_expansion: Optional[int]
    ID_expansion_version: Optional[int]
    ID_expansion: Optional[int]
    pan: Optional[int]
    address: Optional[int]
    ID_org: Optional[int]
    ID_station_type: Optional[int]
    serial: Optional[int]
    code: Optional[str]
    active: Optional[int]
    token: Optional[int]
    location: Optional[str]
    number: Optional[str]
    Analog: Optional[AnalogClass]

    def __init__(self, ID_AUX_station: int, ID_station: int, title: str, output: int, type: int, expansion: int,
                 local: int, settings: None, created_at: datetime, updated_at: datetime, ID_org: Optional[int],
                 ID_station_type: Optional[int], serial: Optional[int], code: Optional[str], active: Optional[int],
                 token: Optional[int], location: Optional[str], number: Optional[str],
                 Analog: Optional[AnalogClass] = None,
                 ID_aux_expansion: Optional[int] = 0, ID_expansion_version: Optional[int] = 0,
                 ID_expansion: Optional[int] = 0, pan: Optional[int] = 0, address: Optional[int] = 0,
                 stationTitle: str = '') -> None:
        self.ID_AUX_station = ID_AUX_station
        self.ID_station = ID_station
        self.stationTitle = stationTitle
        self.title = title
        self.output = output
        self.type = type
        self.expansion = expansion
        self.local = local
        self.settings = settings
        self.created_at = created_at
        self.updated_at = updated_at
        self.ID_aux_expansion = ID_aux_expansion
        self.ID_expansion_version = ID_expansion_version
        self.ID_expansion = ID_expansion
        self.pan = pan
        self.address = address
        self.ID_org = ID_org
        self.ID_station_type = ID_station_type
        self.serial = serial
        self.code = code
        self.active = active
        self.token = token
        self.location = location
        self.number = number
        if Analog != None:
            self.Analog = AnalogClass(**dict(Analog))


class AuxStationData:
    aux: Aux

    def __init__(self, data) -> None:
        self.aux = Aux(**dict(data))


def AuxOnIDLocal(AuxData: List[Aux], ID_aux):
    for aux in AuxData:
        if int(aux.ID_AUX_station) == int(ID_aux):
            return aux

    return None


def AuxOnNumberLocal(AuxData: List[Aux], number):
    for aux in AuxData:
        if str(aux.number) == str(number):
            return aux

    return None



from sdsRayanArvin.Api import Api
from sdsRayanArvin.Request import Request


class Factory:
    """python samacontrol architecture"""

    def __init__(self, api='Old', Url=None):
        print('create instance and config api: ' + api)
        self.RequestApi = None
        self.apiName = None
        self.__ApiConnected = self.__API(api, Url)

    def __API(self, api='Old', Url=None):
        if self.RequestApi is not None:
            return self.RequestApi

        if api == 'Old':
            request = Request('https://samacontrol.ir/sama/api/')

        if api == 'Data':
            request = Request('https://samacontrol.ir/device_data_managment/')

        if api == 'Workflow':
            request = Request('https://samacontrol.ir/workflow-service/')

        if api == 'Rule':
            request = Request('https://samacontrol.ir/rule-service/')

        if api == 'Weather':
            request = Request('https://samacontrol.ir/weather/')

        if api == 'RuleExe':
            request = Request('https://samacontrol.ir/rule-exe/')

        if api == 'WorkflowsExe':
            request = Request('https://samacontrol.ir/workflow-exe/')

        if api == 'Log':
            request = Request('http://49a79b8b-022f-487a-86f6-dfb43132b8b0.hsvc.ir:30122/')

        self.apiName = api
        self.RequestApi = request

        # connect to API
        return self.__connectToApi()

    def __connectToApi(self):
        if self.RequestApi is None:
            print('please set API then connect to API.')

        return Api(self.RequestApi)

    def connectToRepo(self, repository):
        return self.__ApiConnected.instanceOfRepository(repository)


# user = Factory()
# authenticate = user.connectToRepo(Authenticate)
# authenticate.Login()
#
# object = user.connectToRepo(Object)
# ruleApi = Factory('Rule')
# ruleApi.RequestApi.setToken(user.RequestApi.getToken())
# rules: Rule = ruleApi.connectToRepo(Rule)
# rules.allRule()
#
# data = Factory('Data')
# data.RequestApi.setToken(user.RequestApi.getToken())
# lastData: Data = data.connectToRepo(Data)
#
# aux: AuxStation = user.connectToRepo(AuxStation)
#
# Pipeline().GetLastData(aux.AuxOnID('489'), lastData)
# #
# lastData.StoreDataOnTopic({
#     'topic': 1,
#     'type': 'DO',
#     'data': '1',
#     'pan': 0,
#     'add': 0,
# })
# x = lastData.LastDataOnTopic('866600042708784', 'DO', '0', '0')
# print(x.getType())
# x = authenticate.UserInformation()

#
# obj: Object = user.connectToRepo(Object)
# obj.ObjectByIdOrg()
# print(obj.ObjectByIdOrg()[0].getTitle())

# workflow = Factory('Workflow')
# workflow.RequestApi.setToken(user.RequestApi.getToken())
# lastData = workflow.connectToRepo(Workflows)
# lastData.dataAll()

# log = Factory('LogApi')
# logwp: LogApi = log.connectToRepo(LogApi)
# logwp.WorkflowLog('1367', '0', '17')



from typing import List, Optional
from enum import Enum
from datetime import datetime


class Output:
    ID_process_list_commands_output: int
    ID_process_list_commands: int
    ID_outputs: int
    output_active: bool

    def __init__(self, ID_process_list_commands_output: int, ID_process_list_commands: int, ID_outputs: int,
                 output_active: bool) -> None:
        self.ID_process_list_commands_output = ID_process_list_commands_output
        self.ID_process_list_commands = ID_process_list_commands
        self.ID_outputs = ID_outputs
        self.output_active = output_active


class Percent:
    ID_process_list_commands_percent: int
    ID_process_list_commands: int
    time: int
    active_percent: int

    def __init__(self, ID_process_list_commands_percent: int, ID_process_list_commands: int, time: int,
                 active_percent: int) -> None:
        self.ID_process_list_commands_percent = ID_process_list_commands_percent
        self.ID_process_list_commands = ID_process_list_commands
        self.time = time
        self.active_percent = active_percent


class Command:
    ID_process_list_commands: int
    ID_process_list: int
    ID_stations: int
    ID_expansions: int
    visual_outputs: bool
    communication: str
    output: List[Output] = []
    percent: Optional[Percent]

    def __init__(self, ID_process_list_commands: int, ID_process_list: int, ID_stations: int, ID_expansions: int,
                 visual_outputs: bool, communication: str, output: List[Output], percent: Optional[Percent]) -> None:
        self.ID_process_list_commands = ID_process_list_commands
        self.ID_process_list = ID_process_list
        self.ID_stations = ID_stations
        self.ID_expansions = ID_expansions
        self.visual_outputs = visual_outputs
        self.communication = communication
        self.output = []
        for param in output:
            self.output.append(Output(**dict(param)))
        if percent:
            self.percent = Percent(**dict(percent))


class Delay:
    ID_process_list_delay: int
    ID_process_list: int
    delay: int

    def __init__(self, ID_process_list_delay: int, ID_process_list: int, delay: int) -> None:
        self.ID_process_list_delay = ID_process_list_delay
        self.ID_process_list = ID_process_list
        self.delay = delay


class CommunicationType(Enum):
    SMS = "sms"
    MQTT = "mqtt"


class Communicate:
    ID_communications: int
    ID_process_list: int
    to: str
    command: str
    type: CommunicationType

    def __init__(self, ID_communications: int, ID_process_list: int, to: str, command: str,
                 type: CommunicationType) -> None:
        self.ID_communications = ID_communications
        self.ID_process_list = ID_process_list
        self.to = to
        self.command = command
        self.type = type


class ProcessListType(Enum):
    COMMAND = "command"
    DELAY = "delay"
    PERCENT = "percent"
    RULE = "rule"
    VISUAL_COMMAND = "visual_command"


class Rule:
    ID_process_list_rule: int
    ID_process_list: int
    ID_rules: int

    def __init__(self, ID_process_list_rule: int, ID_process_list: int, ID_rules: int) -> None:
        self.ID_process_list_rule = ID_process_list_rule
        self.ID_process_list = ID_process_list
        self.ID_rules = ID_rules


class ProcessList:
    ID_process_list: int
    ID_workflows: int
    process_list_type: ProcessListType
    position: int
    created_at: datetime
    updated_at: datetime
    command: Optional[Command]
    rule: Optional[Rule]
    delay: Optional[Delay]
    communicate: Optional[Communicate]

    def __init__(self, ID_process_list: int, ID_workflows: int, process_list_type: ProcessListType, position: int,
                 created_at: datetime, updated_at: datetime, command: Optional[Command], rule: Optional[Rule],
                 delay: Optional[Delay], communicate: Optional[Communicate]) -> None:
        self.ID_process_list = ID_process_list
        self.ID_workflows = ID_workflows
        self.process_list_type = process_list_type
        self.position = position
        self.created_at = created_at
        self.updated_at = updated_at
        if command:
            self.command = Command(**dict(command))
        if rule:
            self.rule = Rule(**dict(rule))
        if delay:
            self.delay = Delay(**dict(delay))
        if communicate:
            self.communicate = Communicate(**dict(communicate))


class WorkflowType(Enum):
    READ = "write"
    WRITE = "read"


class Workflow:
    ID_workflows: int
    ID_org: int
    title: str
    type: WorkflowType
    created_at: datetime
    updated_at: datetime
    deleted_at: None
    process_lists: List[ProcessList] = []

    def __init__(self, ID_workflows: int, ID_org: int, title: str, type: WorkflowType, created_at: datetime,
                 updated_at: datetime,
                 deleted_at: None, process_lists: List[ProcessList]) -> None:
        self.ID_workflows = ID_workflows
        self.ID_org = ID_org
        self.title = title
        self.type = type
        self.created_at = created_at
        self.updated_at = updated_at
        self.deleted_at = deleted_at
        self.process_lists = []
        for param in process_lists:
            self.process_lists.append(ProcessList(**dict(param)))


class WorkflowsData:
    data: Workflow

    def __init__(self, data) -> None:
        self.data = Workflow(**dict(data))


def WorkflowsOnIDLocal(WorkflowsData: List[Workflow], ID_workflows):
    for workflows in WorkflowsData:
        if int(workflows.ID_workflows) == int(ID_workflows):
            return workflows

    return None



from sdsRayanArvin.Dataset.WorkflowsData import WorkflowsData
from sdsRayanArvin.Repository import Repository


class Workflows(Repository):
    def dataAll(self):
        res = self.Request.sendWithToken('GET', 'workflow_data')
        try:
            workflows = []
            for workflow in res['success']:
                workflows.append(WorkflowsData(workflow).data)
            return workflows
        except:
            return []

    def dataOnID(self, ID_workflows):
        res = self.Request.sendWithToken('GET', 'workflow_data/' + ID_workflows)
        try:
            return WorkflowsData(res['success']).data
        except:
            return []



import json

import requests

from sdsRayanArvin.Cache import Cache
from sdsRayanArvin.Token import Token


class Request(Token, Cache):

    def __init__(self, URL):
        super().__init__()
        Cache.__init__(self)
        Token.__init__(self)
        self.Url = URL

    def sendWithToken(self, method, url, body={}, typeBody='form'):
        if typeBody == 'form':
            res = requests.request(method, self.Url + url, data=body, headers={"Authorization": "bearer " + self.getToken()})
        else:
            res = requests.request(method, self.Url + url, json=body, headers={"Authorization": "bearer " + self.getToken()})
        try:
            return json.loads(res.text)
        except:
            return str(res.status_code)

    def sendWithOutToken(self, method, url, body={}, typeBody='form'):
        if typeBody == 'form':
            res = requests.request(method, self.Url + url, data=body)
        else:
            res = requests.request(method, self.Url + url, json=body)

        try:
            return json.loads(res.text)
        except:
            return str(res.status_code)



from sdsRayanArvin.Dataset import WeatherData
from sdsRayanArvin.Repository import Repository


class Weather(Repository):
    def WeatherOnFarm(self, ID_farm):
        res = self.Request.sendWithToken('GET', 'farm/' + ID_farm)
        try:
            return WeatherData(res['success'])
        except:
            return []



from sdsRayanArvin.Request import Request


class Api:

    def __init__(self, requestClass: Request):
        self.Request = requestClass

    def instanceOfRepository(self, repositoryClass):
        """
        :return: :class:`repositoryClass <repositoryClass>` object
        """
        return repositoryClass(self.Request)



from sdsRayanArvin.Dataset.RuleData import RuleData
from sdsRayanArvin.Repository import Repository


class Rule(Repository):
    def allRule(self):
        res = self.Request.sendWithToken('GET', 'rule_group')
        try:
            return RuleData(res['success']).data
        except:
            return []



from sdsRayanArvin.Dataset.AuxStationData import AuxStationData
from sdsRayanArvin.Repository import Repository


class AuxStation(Repository):
    def AuxAll(self):
        res = self.Request.sendWithToken('GET', 'aux_all')
        auxs = []
        try:
            for aux in res['Aux']:
                auxs.append(AuxStationData(aux).aux)
            return auxs
        except:
            return auxs

    def AuxOnID(self, ID_AUX_station):
        res = self.Request.sendWithToken('GET', 'aux/' + ID_AUX_station)
        try:
            return AuxStationData(res['Aux']).aux
        except:
            return []



from enum import Enum
from typing import Optional, List
from datetime import datetime


class Condition(Enum):
    AND = "and"
    OR = "or"


class Operation(Enum):
    BIGGER = "bigger"
    EQUAL = "equal"
    SMALLER = "smaller"


class RuleStation:
    ID_rule_station: int
    ID_rule_task: int
    ID_outputs: int

    def __init__(self, ID_rule_station: int, ID_rule_task: int, ID_outputs: int) -> None:
        self.ID_rule_station = ID_rule_station
        self.ID_rule_task = ID_rule_task
        self.ID_outputs = ID_outputs


class RuleTime:
    ID_rule_time: int
    ID_rule_task: int
    type: str

    def __init__(self, ID_rule_time: int, ID_rule_task: int, type: str) -> None:
        self.ID_rule_time = ID_rule_time
        self.ID_rule_task = ID_rule_task
        self.type = type


class RuleWeatherType(Enum):
    TEMPERATURE = "temperature"
    WIND_SPEED = "windSpeed"


class RuleWeather:
    ID_rule_weather: int
    ID_rule_task: int
    ID_farm: int
    type: RuleWeatherType

    def __init__(self, ID_rule_weather: int, ID_rule_task: int, ID_farm: int, type: RuleWeatherType) -> None:
        self.ID_rule_weather = ID_rule_weather
        self.ID_rule_task = ID_rule_task
        self.ID_farm = ID_farm
        self.type = type


class RuleTaskType(Enum):
    STATION = "station"
    TIME = "time"
    WEATHER = "weather"


class RuleTask:
    ID_rule_task: int
    ID_rule_group: int
    type: RuleTaskType
    condition: Optional[Condition]
    operation: Operation
    value: str
    rule_station: Optional[RuleStation]
    rule_weather: Optional[RuleWeather]
    rule_time: Optional[RuleTime]

    def __init__(self, ID_rule_task: int, ID_rule_group: int, type: RuleTaskType, condition: Optional[Condition], operation: Operation, value: str, rule_station: Optional[RuleStation], rule_weather: Optional[RuleWeather], rule_time: Optional[RuleTime]) -> None:
        self.ID_rule_task = ID_rule_task
        self.ID_rule_group = ID_rule_group
        self.type = type
        self.condition = condition
        self.operation = operation
        self.value = value
        if rule_station:
            self.rule_station = RuleStation(**dict(rule_station))
        if rule_weather:
            self.rule_weather = RuleWeather(**dict(rule_weather))
        if rule_time:
            self.rule_time = RuleTime(**dict(rule_time))


class RuleWorkflow:
    ID_rule_workflows: int
    ID_rule_group: int
    ID_workflows: int

    def __init__(self, ID_rule_workflows: int, ID_rule_group: int, ID_workflows: int) -> None:
        self.ID_rule_workflows = ID_rule_workflows
        self.ID_rule_group = ID_rule_group
        self.ID_workflows = ID_workflows


class RuleType(Enum):
    PROCESS = "process"
    RULE = "rule"


class Rule:
    ID_rule_group: int
    ID_org: int
    title: str
    active: int
    parent: Optional[int]
    type: RuleType
    created_at: datetime
    updated_at: datetime
    deleted_at: None
    rule_task: List[RuleTask] = []
    rule_workflows: List[RuleWorkflow] = []
    parents = []

    def __init__(self, ID_rule_group: int, ID_org: int, title: str, active: int, parent: int, type: RuleType, created_at: datetime, updated_at: datetime, deleted_at: None, rule_task: List[RuleTask], rule_workflows: List[RuleWorkflow], parents=[]) -> None:
        self.ID_rule_group = ID_rule_group
        self.ID_org = ID_org
        self.title = title
        self.active = active
        self.parent = parent
        self.type = type
        self.created_at = created_at
        self.updated_at = updated_at
        self.deleted_at = deleted_at
        self.rule_task = []
        self.rule_workflows = []
        self.parents = []
        for param in rule_task:
            self.rule_task.append(RuleTask(**dict(param)))
        for param in rule_workflows:
            self.rule_workflows.append(RuleWorkflow(**dict(param)))

        self.parents: List[Rule] = RuleData(parents).data


class RuleData:
    data: List[Rule] = []

    def __init__(self, data) -> None:
        self.data = []
        for param in data:
            self.data.append(Rule(**dict(param)))



class DeviceData:
    def __init__(self, dataArray):
        self.__topic = dataArray['topic']
        self.__type = dataArray['type']
        self.__data = dataArray['data']
        self.__pan = dataArray['pan']
        self.__add = dataArray['add']
        self.__IdDeviceData = dataArray['ID_device_data']

    def getIdDeviceData(self):
        return self.__IdDeviceData

    def getType(self):
        return self.__type

    def getTopic(self):
        return self.__topic

    def getData(self):
        return self.__data

    def getPan(self):
        return self.__pan

    def getAdd(self):
        return self.__add



from sdsRayanArvin.Repository import Repository


class WorkflowsExe(Repository):
    def ProcessRunOnId(self, ID_process):
        res = self.Request.sendWithToken('GET', 'run/' + ID_process + '/0')
        try:
            return res['status']
        except:
            return False



from sdsRayanArvin.Repository import Repository


class RuleExe(Repository):
    def RuleRunOnId(self, ID_rule):
        res = self.Request.sendWithToken('GET', 'run/' + ID_rule)
        try:
            return res['status']
        except:
            return False
